// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Null driver -- the example
//

///////////////////////////////////////////////////////////////////////////////
// Symbols to be escaped in queries. May be defined before #include'ing
// DrvHeader.h -- it's used there. By default "\\'\"" used.
//
#define	EscapedChars	"\\'"

///////////////////////////////////////////////////////////////////////////////
// Including standart headers and functions
//
#include "DrvHeader.h"
#include <string.h>

///////////////////////////////////////////////////////////////////////////////
// Data strcuture for driver.
//
struct DS(data) : public easysql_error_reporting
{ char const *foo;
  unsigned bar;
  bool connected;

  DS(data)();
  ~DS(data)();
};

///////////////////////////////////////////////////////////////////////////////
// If driver's data have to be constructed it should be done here
//
DS(data)::DS(data) ()
         :easysql_error_reporting(DriverName "_driver",D_IDRV)
         ,foo("bar")
         ,bar(0xF00)
         ,connected(false)
{
}

///////////////////////////////////////////////////////////////////////////////
// If driver's data have to be destructed it should be done here
//
DS(data)::~DS(data) ()
{
}

///////////////////////////////////////////////////////////////////////////////
// Version information for driver
//
char const *
DS(version) (DS(data) *)
{ return DriverName " " EASYSQL_VERSION;
}

///////////////////////////////////////////////////////////////////////////////
// Connecting to engine. For null driver always return success.
//
bool
DS(connect) (DS(data) *data, char const *(*values)(void const *, char const *), void const *that)
{ log(D_IDRV|D_L4,"`%s' driver: connect()",DriverName);
  log(D_IDRV|D_L1,"Some connect() parameters:");
  log(D_IDRV|D_L1," host=     %s",values(that,"host"));
  log(D_IDRV|D_L1," db=       %s",values(that,"database"));
  log(D_IDRV|D_L1," user=     %s",values(that,"user"));
  log(D_IDRV|D_L1," password= %s",values(that,"password"));
  return (data->connected=true);
}

///////////////////////////////////////////////////////////////////////////////
// Dropping connection. Used by driver manager as a way to disconnect
// from SQL engine by timeout or by direct request from user.
//
bool
DS(drop) (DS(data) *data)
{ log(D_IDRV|D_L4,"`%s' driver: drop(..)",DriverName);
  data->connected=false;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Sending query to SQL engine. In null driver always return success.
// It is not reqiured to check arguments for their consistency - they
// are always fine - EasySQL driver manager checks it.
//
bool
DS(query) (DS(data) *, esqlResult &r, char const *q)
{ log(D_IDRV|D_L4,"`%s' driver: query(..,..,\"%s\")",DriverName,q);
  if(!strncasecmp(q,"select ",7))
   { r=(esqlResult)new unsigned(3);
     log(D_IDRV|D_L4,"`%s' driver: query() returning %p",DriverName,r);
     return r ? true : false;
   }
  r=NULL;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Selecting next row in sequence of data, returned from query.
// When there are no more rows to be returned NULL is returned in res.
// Return value only shows how successfull was function - and even if
// there is no more data it may (and should) be true.
//
bool
DS(row) (DS(data) *, esqlRow &row, esqlResult r)
{ log(D_IDRV|D_L4,"`%s' driver: row(%p)",DriverName,r);
  if(r && *((unsigned *)r))
   { static char const *text="null-test";
     row=&text;
     (*((unsigned *)r))--;
     return true;
   }
  row=NULL;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Frees data, returned by query. A must for every query after fetcing
// all interesting data.
// Not issued by manager automatically, but only by direct request from
// user program.
//
bool
DS(rfree) (DS(data) *, esqlResult)
{ log(D_IDRV|D_L4,"`%s' driver: rfree(..)",DriverName);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking connection state
// Is always true for `null' driver
//
bool
DS(check_connection) (DS(data) *data)
{ return data->connected;
}

///////////////////////////////////////////////////////////////////////////////
// Checking and returning SQL engine and/or connection status. One line in
// free format without trailing '\n'
//
char const *
DS(status) (DS(data) *)
{ return "`null' driver is always ready to do fake operations :)";
}

///////////////////////////////////////////////////////////////////////////////
// Standart create/finish/error functions
//
#include "../DrvStd.h"

///////////////////////////////////////////////////////////////////////////////
R("$Id: null.C,v 1.3 1997/09/20 19:05:36 am Exp $");
